@@ -10,7 +10,7 @@
 #   IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 #   WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
-#   $Id: eputil.c,v 1.23 2001/06/05 03:22:09 richter Exp $
+#   $Id: eputil.c,v 1.26 2001/09/01 21:31:10 richter Exp $
 #
 ###################################################################################*/
 
@@ -129,6 +129,82 @@ void OutputEscape (/*i/o*/ register req * r,
         owrite (r, p, sData - p) ;
     }
 
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* Escape a string and return a sv                                              */
+/*                                                                              */
+/* in sData     = input:  string                                                */
+/*    nDataLen  = input:  length of string                                      */
+/*    pEscTab   = input:  escape table                                          */
+/*    cEscChar  = input:  char to escape escaping (0 = off)                     */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
+
+SV * Escape	  (/*i/o*/ register req * r,
+ 		   /*in*/  const char *   sData,
+ 		   /*in*/  int            nDataLen,
+ 		   /*in*/  int            nEscMode,
+ 		   /*in*/  struct tCharTrans *   pEscTab,
+ 		   /*in*/  char           cEscChar)
+
+    {
+    char * pHtml  ;
+    const char * p ;
+    int	         l ;
+    SV *         pSV = newSVpv("",0) ;
+
+    EPENTRY (Escape) ;
+
+
+    if (nEscMode >= 0)
+	{	    
+	if (nEscMode & escHtml && !r -> bEscInUrl)
+	    pEscTab = Char2Html ;
+	else if (nEscMode & escUrl)
+	    pEscTab = Char2Url ;
+	else 
+	    pEscTab = NULL ;
+	if (nEscMode & escEscape)
+	    cEscChar = '\0' ;
+	else
+	    cEscChar = '\\' ;
+	}
+
+    if (pEscTab == NULL)
+        {
+        sv_setpvn (pSV, sData, nDataLen) ;
+        return pSV ;
+        }
+
+    p = sData ;
+    l = nDataLen ;
+
+    while (l > 0)
+        {
+        if (cEscChar && *sData == cEscChar)
+            {
+            if (p != sData)
+		sv_catpvn (pSV, (char *)p, sData - p) ;
+            sData++, l-- ;
+            p = sData ;
+            }
+        else
+            {
+            pHtml = pEscTab[(unsigned char)(*sData)].sHtml ;
+            if (*pHtml)
+                {
+                if (p != sData)
+                    sv_catpvn (pSV, (char *)p, sData - p) ;
+                sv_catpv (pSV, pHtml) ;
+                p = sData + 1;
+                }
+            }
+        sData++, l-- ;
+        }
+    if (p != sData)
+        sv_catpvn (pSV, (char *)p, sData - p) ;
+    return pSV ;
+    }
 
 #if 0
 
@@ -315,6 +391,23 @@ int   TransHtml (/*i/o*/ register req * r,
 	return nLen ; 	
         }
         
+#ifdef EP2
+    if (bInUrl == 16)
+	{ 
+	/* Just remove \ for rtf */
+	if (nLen == 0)
+	    nLen = strlen (sData) ;
+	e = sData + nLen ;
+	while (p < e)
+	    {
+	    if (*p == '\\' && p[1] != '\0')
+	    	*p++ = ' ' ;
+	    p++ ;
+	    }	
+	return nLen ; 	
+        }
+#endif
+
     s = NULL ;
     if (nLen == 0)
         nLen = strlen (sData) ;
@@ -961,10 +1054,10 @@ void ClearSymtab (/*i/o*/ register req * r,
     (void)hv_iterinit(symtab);
     while ((val = hv_iternextsv(symtab, &key, &klen))) 
 	{
-	if(SvTYPE(val) != SVt_PVGV)
+	if(SvTYPE(val) != SVt_PVGV || SvANY(val) == NULL)
 	    {
 	    if (bDebug)
-	        lprintf (r, "[%d]CUP: Ignore ??? because it's no gv\n", r -> nPid) ;
+	        lprintf (r, "[%d]CUP: Ignore %s because it's no gv\n", r -> nPid, key) ;
 	    
 	    continue;
 	    }
@@ -1011,7 +1104,7 @@ void ClearSymtab (/*i/o*/ register req * r,
 	
 	sObjName = NULL ;
 	
-        lprintf (r, "[%d]CUP: type = %d flags=%x\n", r -> nPid, SvTYPE (GvSV((GV*)val)), SvFLAGS (GvSV((GV*)val))) ;
+        /* lprintf (r, "[%d]CUP: type = %d flags=%x\n", r -> nPid, SvTYPE (GvSV((GV*)val)), SvFLAGS (GvSV((GV*)val))) ; */
         if((sv = GvSV((GV*)val)) && SvTYPE (sv) == SVt_PVMG)
 	    {
             HV * pStash = SvSTASH (sv) ;
@@ -1034,7 +1127,7 @@ void ClearSymtab (/*i/o*/ register req * r,
         if((sv = GvSV((GV*)val)) && SvROK (sv) && SvOBJECT (SvRV(sv)))
 	    {
             HV * pStash = SvSTASH (SvRV(sv)) ;
-        lprintf (r, "[%d]CUP: rv type = %d\n", r -> nPid, SvTYPE (SvRV(GvSV((GV*)val)))) ;
+	    /* lprintf (r, "[%d]CUP: rv type = %d\n", r -> nPid, SvTYPE (SvRV(GvSV((GV*)val)))) ; */
             if (pStash)
                 {
                 sObjName = HvNAME(pStash) ;
@@ -1128,3 +1221,54 @@ void UndefSub    (/*i/o*/ register req * r,
     cv_undef (pCV) ;
     }
 
+
+/* ---------------------------------------------------------------------------- */
+/*                                                                              */
+/* Get Session ID                                                               */
+/*                                                                              */
+/* ---------------------------------------------------------------------------- */
+
+
+char * GetSessionID (/*i/o*/ register req * r,
+   		     /*in*/  HV * pSessionHash,
+		     /*out*/ char * * ppInitialID,
+		     /*out*/ IV * pModified)
+    
+    {
+    SV **   ppSVID ;
+    SV *    pSVID = NULL ;
+    MAGIC * pMG ;
+    char *  pUID = "" ;
+    STRLEN  ulen = 0 ;
+    STRLEN  ilen = 0 ;
+
+    if (r -> nSessionMgnt)
+	{			
+	SV * pUserHashObj = NULL ;
+	if ((pMG = mg_find((SV *)pSessionHash,'P')))
+	    {
+	    dSP;                            /* initialize stack pointer      */
+	    int n ;
+	    pUserHashObj = pMG -> mg_obj ;
+
+	    PUSHMARK(sp);                   /* remember the stack pointer    */
+	    XPUSHs(pUserHashObj) ;            /* push pointer to obeject */
+	    PUTBACK;
+	    n = perl_call_method ("getids", G_ARRAY) ; /* call the function             */
+	    SPAGAIN;
+	    if (n > 2)
+		{
+		int  savewarn = dowarn ;
+		dowarn = 0 ; /* no warnings here */
+		*pModified = POPi ;
+		pSVID = POPs;
+		pUID = SvPV (pSVID, ulen) ;
+		pSVID = POPs;
+		*ppInitialID = SvPV (pSVID, ilen) ;
+		dowarn = savewarn ;
+		}
+	    PUTBACK;
+	    }
+        }
+    return pUID ;
+    }
\ No newline at end of file
